#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <string.h>


#define logit(x) ( log( (x) / ( 1.0 - (x) ) ) )
#define inv_logit(x) ( 1.0 / ( 1.0 + exp(-(x)) ) )

#include "matrix.h"
#include "datastruct.h" /*specifying data structure*/
#include "config.h" /*read in configuration parameters for flu and trial*/

int main()
{
  int i, j, k, l, m, n, len, len_inc, len_inf;
  int n_inc, n_inf;
  int *min_inc, *max_inc, *lower_inf, *upper_inf;
  double **prob_inc, **prob_inf;
  FILE * file;
  char c, st[100];
  CFG_PARS cfg_pars;

  if((file = fopen("config.file", "r")) == NULL)
  {
    printf("couldn't open config.file\n");
    exit(0);
  }
  cfg_pars = get_cfg_pars(file, 0);
  rewind(file);

  while( ! feof(file))
  {
     c=fgetc(file);
     if(c == '#')
     {
        fgets(st, 100, file);
        if(strstr(st, "min-max-days-and-probs-of-incubation-period")!=NULL)
        {
           fscanf(file,"%d", &n_inc);
           min_inc = (int *) malloc((size_t) (n_inc * sizeof(int)));
           max_inc = (int *) malloc((size_t) (n_inc * sizeof(int)));
           prob_inc = (double **) malloc((size_t) (n_inc * sizeof(double)));
           
           for(i=0; i<n_inc; i++)
           {
              fscanf(file, "%d  %d", &min_inc[i], &max_inc[i]);
              //printf("%d  %d\n", min_inc[i], max_inc[i]);
              len = max_inc[i] - min_inc[i] + 1;
              prob_inc[i] = (double *)malloc((size_t) (len * sizeof(double)));
              for(j = 0 ; j < len ; j++)
              {
                 fscanf(file, "%lf", prob_inc[i] + j);
                 //printf("%lf  ", prob_inc[i][j]);
              }
              //printf("\n");
           }
        }

        if(strstr(st, "primary-lower-upper-bounds-and-probs-of-infectious-days-relative-to-symptom-onset-day")!=NULL)
        {
           fscanf(file,"%d", &n_inf);
           lower_inf = (int *) malloc((size_t) (n_inf * sizeof(int)));
           upper_inf = (int *) malloc((size_t) (n_inf * sizeof(int)));
           prob_inf = (double **) malloc((size_t) (n_inf * sizeof(double)));
           
           for(i=0; i<n_inf; i++)
           {
              fscanf(file, "%d  %d", &lower_inf[i], &upper_inf[i]);
              //printf("%d  %d\n", lower_inf[i], upper_inf[i]);
              len = upper_inf[i] - lower_inf[i] + 1;
              prob_inf[i] = (double *) malloc((size_t) (len * sizeof(double)));
              for(j = 0 ; j < len ; j++)
              {
                 fscanf(file, "%lf", prob_inf[i] + j);
                 //printf("%lf  ", prob_inf[i][j]);
              }
              //printf("\n");
           }
        }
     }
  }

  fclose(file);   /* close the file handle */

  if( fopen("estimates.txt", "r") != NULL )  {fclose(file); system("del -f estimates.txt");}
  if( fopen("errors.txt", "r") != NULL )  {fclose(file); system("del -f errors.txt");}
  for(i=0; i<n_inc; i++)
  {
     cfg_pars.max_incubation = max_inc[i];
     cfg_pars.min_incubation = min_inc[i];
     len_inc = max_inc[i] - min_inc[i] + 1;
     cfg_pars.prob_incubation = (double *)malloc((size_t) (len_inc * sizeof(double)));
     for(k = 0 ; k < len_inc ; k++)
        cfg_pars.prob_incubation[k] = prob_inc[i][k];
     for(j=0; j<n_inf; j++)
     {
        cfg_pars.upper_infectious = upper_inf[j];
        cfg_pars.lower_infectious = lower_inf[j];
        len_inf = upper_inf[j] - lower_inf[j] + 1;
        cfg_pars.prob_infectious = (double *)malloc((size_t) (len_inf * sizeof(double)));
        for(l = 0 ; l < len_inf ; l++)
           cfg_pars.prob_infectious[l] = prob_inf[j][l];
        if(cfg_pars.effective_bounds_provided == 0)
        {
           for(l=0; l<cfg_pars.n_p_mode; l++)
           {
              cfg_pars.effective_lower_infectious[l] = cfg_pars.lower_infectious; 
              cfg_pars.effective_upper_infectious[l] = cfg_pars.upper_infectious; 
           }           
        }
        file = fopen("config.tmp", "w");
        write_cfg_pars(file, &cfg_pars);
        fclose(file);
        
        if((file = fopen("estimates.txt", "a")) == NULL)
           file = fopen("estimates.txt", "w");
        fprintf(file, "\n\n=======================================================================================\n");
        fprintf(file, "%d: min_inc=%d  max_inc=%d  prob_inc=(", i+1, cfg_pars.min_incubation, cfg_pars.max_incubation);
        for(k = 0 ; k < len_inc ; k++)  fprintf(file, "%5.3lf, ", cfg_pars.prob_incubation[k]);
        fprintf(file, ")\n");
        fprintf(file, "\n%d: lower_inf=%d  upper_inf=%d  prob_inf=(", j+1, cfg_pars.lower_infectious, cfg_pars.upper_infectious);
        for(l = 0 ; l < len_inf ; l++)  fprintf(file, "%5.3lf, ", cfg_pars.prob_infectious[l]);
        fprintf(file, ")\n");
        fprintf(file, "=======================================================================================\n");
        fclose(file);
        system("main.exe");

     }  
  }   
  free(min_inc);
  free(max_inc);
  free(lower_inf);
  free(upper_inf);
  for(i=0; i<n_inc; i++)  free(prob_inc[i]);
  free(prob_inc);
  for(i=0; i<n_inf; i++)  free(prob_inf[i]);
  free(prob_inf);

  free(cfg_pars.prob_incubation);
  free(cfg_pars.prob_infectious);
  free(cfg_pars.c2p_covariate);
  free(cfg_pars.sus_p2p_covariate);
  free(cfg_pars.inf_p2p_covariate);
  free(cfg_pars.pat_covariate);
  free(cfg_pars.imm_covariate);
  free_2d_array_int(cfg_pars.interaction);
  free_2d_array_double(cfg_pars.ini_par_effective);
  free(cfg_pars.lower_search_bound);
  free(cfg_pars.upper_search_bound);
  free(cfg_pars.converge_criteria);
  free(cfg_pars.R0_multiplier);
  free(cfg_pars.R0_multiplier_var);
  free(cfg_pars.par_fixed_id);
  free(cfg_pars.par_fixed_value);
  free(cfg_pars.sim_par_effective);
  free(cfg_pars.effective_lower_infectious);
  free(cfg_pars.effective_upper_infectious);
  if(cfg_pars.par_equiclass != NULL)
  {
     for(i=0; i<cfg_pars.n_par_equiclass; i++)
     {
        free(cfg_pars.par_equiclass[i].member);
     }
     free(cfg_pars.par_equiclass);
  }
  return(0);
}
